from setuptools import setup

setup(
	name='prob_distrs',
    version='0.3',
    description='Some popular probability distributions',
    packages=['prob_distrs'],
    author="Aditya Vadlamani",
    author_email = 'aditya.t.vadlamani@gmail.com',
    url = 'https://github.com/AdityaVadlamani/prob-distrs',
    zip_safe=False
)
